/***************************************************************************************************
Daten-Modul fr MSB Sensordaten
****************************************************************************************************
Controller:     -
Taktfrequenz:   -
Entwickler:     Reinhardt Werbik
Version:        1.00
Datum:          29.10.2017
****************************************************************************************************
Dieses Modul enthlt ein Daten-Array zum Ablegen der Sensordaten fr alle 16 mglichen MSB Adressen.
Der Array-Index gibt dabei die dem entsprechenden Datensatz zugeordnete Adresse an.
Jeder Datensatz beinhaltet Werteklasse, Sensorwert, Alarm Flag und Gltigkeit des Sensorwertes.
Ein zustzliches Flag zeigt an, ob der Datensatz aktualisiert wurde.

Das MSB Sensordaten-Array wird als globale Variable verfgbar gemacht.

Das Modul verarbeitet Datenpakete vom M-Link Sendemodul.
Dabei werden aus den empfangenen M-Link Daten die entsprechenden Sensorwerte ermittelt
und zusammen mit dem Alarm Flag an der entsprechenden Stelle im MSB Sensordaten-Array abgelegt.

Der Code in diesem Modul ist unabhngig vom verwendeten Controller und der Taktfrequenz.
****************************************************************************************************
nderungen:

V 1.00:
- Erste konfigurierte Version
***************************************************************************************************/

#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdint.h>
#include <stdbool.h>

#include "mlink_data.h"
#include "mlink_interface.h"

//========== Variablen

msb_data_t msb_data[16];    // MSB Sensordaten-Array (Index = Adresse)

//========== Interne Prototypen

void update_msb_data(uint8_t header, uint8_t low_byte, uint8_t high_byte);

//========== Interne Funktionen

void update_msb_data(uint8_t header, uint8_t low_byte, uint8_t high_byte)
{
	uint8_t addr = header >> 4;
	uint8_t class = header & 0x0F;
	uint8_t value_l = 0;
	uint8_t value_h = 0;

	if ((256 * high_byte) + low_byte == NO_VALUE)
	{
		msb_data[addr].valid = false;       // ungltiger Messwert
	}
	else
	{
		if (class == MSB_SPECIAL)
		{
			if (high_byte == 0x01)
			{
				//ECU Status- und Fehlermeldungen
				msb_data[addr].alarm = (low_byte % 2 == 1);     // Alarm-Flag speichern
				value_l = (low_byte >> 1);                      // Alarm-Flag aus Low Byte entfernen
				msb_data[addr].class = MSB_SPECIAL;
				msb_data[addr].value = value_l;     // Sensorwert ist der 7-Bit Code der ECU Meldung
				msb_data[addr].valid = true;
				msb_data[addr].updated = true;
			}
			else
			{
				msb_data[addr].valid = false;       // Sensor nicht mehr verfgbar
			}
		}
		else
		{
			// normaler Sensorwert (keine Sonderdaten)
			msb_data[addr].alarm = (low_byte % 2 == 1);     // Alarm-Flag speichern
			value_l = (low_byte >> 1);                      // Alarm-Flag aus Low Byte entfernen
			if ((high_byte % 2) == 1)
			{
				value_l += 128;     // gesetztes LSB des High Bytes bercksichtigen
			}
			value_h = (high_byte >> 1);         // LSB aus High Byte entfernen
			value_h |= (high_byte & 0x80);      // gesetztes Vorzeichen-Bit bercksichtigen
			msb_data[addr].class = class;
			msb_data[addr].value = (uint16_t)(value_h << 8) + value_l;
			msb_data[addr].valid = true;
			msb_data[addr].updated = true;
		}
	}
}

//========== Interface Funktionen

void init_msb_data(void)
{
	uint8_t i;

	for (i = 0; i < 16; i++)
	{
		msb_data[i].class = 0;
		msb_data[i].value = 0;
		msb_data[i].alarm = false;
		msb_data[i].valid = false;
		msb_data[i].updated = false;
	}
}

void process_mlink_data(void)
{
	update_msb_data(mlink_byte(8), mlink_byte(9), mlink_byte(10));      // 1. Parameter
	update_msb_data(mlink_byte(11), mlink_byte(12), mlink_byte(13));    // 2. Parameter
}