/***************************************************************************************************
Header fr MSB Daten-Modul
****************************************************************************************************
Entwickler:     Reinhardt Werbik
Version:        1.00
Datum:          29.10.2017
****************************************************************************************************
nderungen:

V 1.00:
- Erste konfigurierte Version
***************************************************************************************************/

#ifndef MLINK_DATA_H
#define MLINK_DATA_H

//========== Konstanten

#define NO_VALUE        0x8000  // Code fr ungltigen Messwert eines MSB Sensors

// MSB Werteklassen (Einheiten)

#define MSB_SPECIAL     0       // Sonderdaten
#define MSB_VOLTAGE     1       // Spannung (0,1 V)
#define MSB_CURRENT     2       // Strom (0,1 A)
#define MSB_VERT_SPEED  3       // Steigrate (0,1 m/s)
#define MSB_SPEED       4       // Geschwindigkeit (0,1 km/h)
#define MSB_RPM         5       // Drehzahl (10 bzw. 100 1/min)
#define MSB_TEMP        6       // Temperatur (0,1 C)
#define MSB_DIRECTION   7       // Richtung (0,1 Grad)
#define MSB_ALTITUDE    8       // Hhe oder kurze Distanz (1 m)
#define MSB_TANK        9       // Fllstand Tank (% Tank)
#define MSB_LQI         10      // Link Quality Indicator (% LQI)
#define MSB_CHARGE      11      // Ladung (1 mAh)
#define MSB_VOLUME      12      // Volumen (1 ml)
#define MSB_DISTANCE    13      // Lange Distanz (0,1 km)
#define MSB_ACCEL       14      // Beschleunigung als g-Rate (0,1 g)

//========== Datentypen

typedef struct
{
	uint8_t class;      // Werteklasse
	int16_t value;      // Sensorwert
	bool alarm;         // Alarm
	bool valid;         // Gltigkeit des Sensorwertes
	bool updated;       // zeigt an, ob der Datensatz aktualisiert wurde
} msb_data_t;

//========== Globale Variablen

extern msb_data_t msb_data[16];     // MSB Sensordaten-Array (Index = Adresse)

//========== Interface Prototypen

void init_msb_data(void);
void process_mlink_data(void);

#endif